/**
  *
  * THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * COPYRIGHT 2007-2012 STMicroelectronics
  *
  ******************************************************************************
  */

#ifndef _MC_
#define _MC_


// ****************************************************************************
// ************************** Configuration Constants *************************
// ****************************************************************************


// ****************************************************************************
// ********************** External Constants and Variables ********************
// ****************************************************************************                

    //MODE constants (used by me_modeTransition())
    #define ME_MODE_RESET    0x00000000
    #define ME_MODE_SAFE     0x20000000
    #define ME_MODE_DRUN     0x30000000
    #define ME_MODE_RUN0     0x40000000
    #define ME_MODE_RUN1     0x50000000
    #define ME_MODE_RUN2     0x60000000
    #define ME_MODE_RUN3     0x70000000
    #define ME_MODE_HALT0    0x80000000
    #define ME_MODE_STOP0    0xA0000000
    #define ME_MODE_STANDBY0 0xD0000000   


    //Constants for Peripheral configuration registers
    //(provides mapping from peripheral name to PCTL register number) 
    #define ME_PCTL_DSPI0       4
    #define ME_PCTL_DSPI1       5
    #define ME_PCTL_DSPI2       6    
   
    #define ME_PCTL_FLEXCAN0    16
#if (PROCESSOR == CPU_SPC560B)    
    #define ME_PCTL_FLEXCAN1    17
    #define ME_PCTL_FLEXCAN2    18
    #define ME_PCTL_FLEXCAN3    19
    #define ME_PCTL_FLEXCAN4    20
    #define ME_PCTL_FLEXCAN5    21
#endif    
    
    #define ME_PCTL_FLEXRAY     24
    
    #define ME_PCTL_SAFETYPORT  26
    
    #define ME_PCTL_ADC0        32
  

#if (PROCESSOR == CPU_SPC560B)
    #define ME_PCTL_I2CDMA0     44
#endif    
    
    #define ME_PCTL_LINFLEX0    48
    #define ME_PCTL_LINFLEX1    49
#if (PROCESSOR == CPU_SPC560B)    
    #define ME_PCTL_LINFLEX2    50
    #define ME_PCTL_LINFLEX3    51

    #define ME_PCTL_CTUL        57

    #define ME_PCTL_CANSAMPLER  60
    
    #define ME_PCTL_CFLASH0     66    
    #define ME_PCTL_DFLASH0     67    
    #define ME_PCTL_SIUL        68
    
    #define ME_PCTL_WKPU        69
    
    #define ME_PCTL_EMIOS0      72
    #define ME_PCTL_EMIOS1      73            
#endif

    
    #define ME_PCTL_SSCM        86    
#if (PROCESSOR == CPU_SPC560B)    
    #define ME_PCTL_MCME        87
    #define ME_PCTL_MCCGM       88
    #define ME_PCTL_MCRGM       89
    #define ME_PCTL_PCU         90
    #define ME_PCTL_RTC         91
#endif        
    #define ME_PCTL_PIT         92
    
#if (PROCESSOR == CPU_SPC560B)    
    #define ME_PCTL_XOSC0       96
        
    #define ME_PCTL_XOSC1       98
    #define ME_PCTL_16MIRC      99
    #define ME_PCTL_128KIRC     100    
    #define ME_PCTL_PLL0        101
        
    #define ME_PCTL_CMU0        104
#endif        
        
                  

// ****************************************************************************
// *********************** Function Prototypes ********************************
// ****************************************************************************
      
// ****************************************************************************
//! @fn          void me_modeTransition (UINT32 modeToEnter)
//! @param
//!              modeToEnter     mode to be entered (see ME_MODE_xxxx constants)
//! @return
//!              none 
//! @brief
//!              Triggers mode transition and waits for its completion
//!
// ****************************************************************************   
    inline void me_modeTransition (UINT32 modeToEnter){
       //Start mode transition       
       //enter to mode (0 = reset, 2 = safe, 3 = drun, 4 = run0, 5 = run1)
       ME.MCTL.R = modeToEnter | 0x00005AF0;
       ME.MCTL.R = modeToEnter | 0x0000A50F;
       
       //wait until this mode transition is finished
       if (modeToEnter < ME_MODE_HALT0) while (ME.GS.B.S_MTRANS);
       ME.IS.R   = 1;            //clear MTC (mode transition completed) bit   
    }//me_modeTransition


    void mc_init(void);

#endif

// *************************** End of mc.h ***************************
