#include "spc560bsp.h"
#include "uart.h"

// Initialization of LINFlex in UART mode
void UART_Init(void) {
    SIU.PCR[18].R = 0x400; // LINFlex0 TX

    LINFLEX_0.LINCR1.B.SLEEP = 0;
    LINFLEX_0.LINCR1.B.INIT  = 1;    // switching LINFlex to initialization mode

    LINFLEX_0.LINCR1.B.MME = 1;      // enabling master mode
    LINFLEX_0.LINFBRR.B.DIV_F = 12;  // setting baudrate to 115200
    LINFLEX_0.LINIBRR.B.DIV_M = 34;

    LINFLEX_0.UARTCR.B.UART = 1;     // enabling UART mode
    LINFLEX_0.UARTCR.B.TXEN = 1;     // only trasmitter enabled in this example
    LINFLEX_0.UARTCR.B.WL = 1;       // 8-bit data

    LINFLEX_0.LINCR1.B.INIT = 0;     // switching LINFlex to normal mode
}

// Putting char in UART
void UART_PutChar(char c) {
    if (c == '\n')
        UART_PutChar('\r');

    LINFLEX_0.BDRL.B.DATA0 = c;
    while (!LINFLEX_0.UARTSR.B.DTF)
        ;
    LINFLEX_0.UARTSR.B.DTF = 1;
}

// Putting string in UART
void UART_PutString(char *p) {
    while (*p)
        UART_PutChar(*p++);
}

// Helpers for hexadecimal outpur
void UART_PutHex(char c) {
    char hi = c >> 4;
    char lo = c & 0x0f;

    hi += (hi > 9) ? 'A' - 10 : '0';
    lo += (lo > 9) ? 'A' - 10 : '0';

    UART_PutChar(hi); UART_PutChar(lo);
}

// Puting hex value of integer variable
void UART_PutInteger(int value) {
    int i;
    for (i = 0; i < 4; i++)
        UART_PutHex((value >> ((3 - i) * 8)) & 0xff);
}
