// *****************************************************************************
// SMET Test Template - linker/locator control file for GHS Multi 4 & 5
// It locates code to ROM and data to RAM, vector table is generated
// There is libc support
// Supported Processor = SPC560B
// *****************************************************************************


MEMORY {

    //  512K Internal Flash
    flash_rsvd1  : ORIGIN = 0x00000000, LENGTH = 4
    flash_rsvd2  : ORIGIN = .,          LENGTH = 4
    flash_memory : ORIGIN = .,          LENGTH = 512K-8-3K
    flash_rsvd3  : ORIGIN = .,          LENGTH = 3K 


    //  32KB of internal SRAM starting at 0x40000000    
    dram_rsvd1   : ORIGIN = 0x40000000, LENGTH = 0
    dram_reset   : ORIGIN = .,          LENGTH = 0
    dram_rsvd2   : ORIGIN = .,          LENGTH = 0
    dram_memory  : ORIGIN = .,          LENGTH = 32K
    
}

CONSTANTS {

    stack_reserve = 8K
    heap_reserve = 8K

}

SECTIONS
{
//Data sections     
    .sdabase                    ALIGN(8)    : > dram_memory 
    .sbss                                   : > .
    .sdata                                  : > .    
    .data                                   : > .
    
    .bss                                    : > .
    .heap     ALIGN(16) PAD(heap_reserve)   : > .   
    .stack  ALIGN(16) PAD(stack_reserve)    : > .
                                        
    //Code sections
    .rchw                               ABS : > flash_rsvd1
    .jmpaddress                         ABS : > flash_rsvd2            
    .text                                   : > flash_memory
    .vletext                                : > .  
    .syscall                                : > .  
    
    .rodata                                 : > .
    
    //section below is used for R/O data when SDA optimization is turned on
    .sdata2                                 : > .
    
    .secinfo                                : > .
    
    //Place for ROM copy of the initialized data which will be copied to RAM 
    //at startup
    .ROM.sdata                  ROM(.sdata) : > .
    .ROM.data                    ROM(.data) : > .
    
    .vtable                             ABS : > flash_rsvd3
   
    //
    // These special symbols mark the bounds of RAM and ROM memory.
    // They are used by the MULTI debugger.
    //
    __ghs_ramstart  = MEMADDR(dram_rsvd1);
    __ghs_ramend    = MEMENDADDR(dram_memory);
    __ghs_romstart  = MEMADDR(flash_rsvd1);
    __ghs_romend    = MEMENDADDR(flash_rsvd3);

}
