// *****************************************************************************
// SMET Test Template - linker/locator control file for GHS Multi 4 & 5
// It locates code to ROM and data to RAM, vector table is generated
// There is no libc support - standard libc sections are not defined here. 
// (this is default file, where whole memory range can be used)
// Supported Processor = SPC560B
// *****************************************************************************


MEMORY {

    //  256kB Internal Flash
    flash_rsvd1  : ORIGIN = 0x00000000, LENGTH = 4
    flash_rsvd2  : ORIGIN = .,          LENGTH = 4
    flash_memory : ORIGIN = .,          LENGTH = 256k-8-3K
    flash_rsvd3  : ORIGIN = .,          LENGTH = 3K 


    // 24KB of internal SRAM starting at 0x40000000
    dram_rsvd1   : ORIGIN = 0x40000000, LENGTH = 0
    dram_reset   : ORIGIN = .,          LENGTH = 0
    dram_rsvd2   : ORIGIN = .,          LENGTH = 0
    dram_memory  : ORIGIN = .,          LENGTH = 24K
    
}

CONSTANTS {

    stack_reserve = 1K
    //heap_reserve = 16K

}

SECTIONS
{
    //Data sections      
    .sdabase                    ALIGN(8)    : > dram_memory     
    .sbss                                   : > .
    .sdata                      ALIGN(8)    : > .
    .data                       ALIGN(8)    : > .
    .bss                                    : > .       
    .stack  ALIGN(16) PAD(stack_reserve)    : > .
                                        

    //Code sections
    .rchw                               ABS : > flash_rsvd1
    .jmpaddress                         ABS : > flash_rsvd2
    .start                                  : > flash_memory            
    .vletext                                : > .    
	._s_vect					ALIGN(4k)	: > .
	._h_vect					ALIGN(2k)	: > .
    .rodata                                 : > .
    
    //section below is used for R/O data when SDA optimization is turned on
    .sdata2                                 : > .
    
    //Place for ROM copy of the initialized data which will be copied to RAM 
    //at startup
    .ROM.sdata                  ALIGN(8) ROM(.sdata) : > .
    .ROM.data                   ALIGN(8) ROM(.data) : > .
        
    .vtable                             ABS : > flash_rsvd3
   
    //
    // These special symbols mark the bounds of RAM and ROM memory.
    // They are used by the MULTI debugger.
    //
    __ghs_ramstart  = MEMADDR(dram_rsvd1);
    __ghs_ramend    = MEMENDADDR(dram_memory);
    __ghs_romstart  = MEMADDR(flash_rsvd1);
    __ghs_romend    = MEMENDADDR(flash_rsvd3);

     //These symbols define where sections with initialized variables are stored     
    __smt_initVariablesStart        = addr(.sdata);
    __smt_initVariablesEnd          = addr(.data)+sizeof(.data);     
    __smt_initVariablesROMCopyStart = addr(.ROM.sdata);
    __smt_initVariablesROMCopyEnd   = addr(.ROM.data)+sizeof(.ROM.data);
	__smt_s_vectStart				= addr(._s_vect);
	__smt_h_vectStart				= addr(._h_vect);
}
