// *****************************************************************************
// SMET Test Template - linker/locator control file for GHS Multi 4 & 5
// It locates both code and data to RAM, vector table is generated
// Supported Processor = SPC560B
// *****************************************************************************

//
//Memory map definition
//
MEMORY {

    // 512K Internal Flash starting at 0x00000000
    flash_rsvd1  : ORIGIN = 0x00000000, LENGTH = 8
    flash_memory : ORIGIN = .,          LENGTH = 512K-8
    flash_rsvd2  : ORIGIN = .,          LENGTH = 0 

    // 32KB of internal SRAM starting at 0x40000000 
    dram_rsvd1      : ORIGIN = 0x40000000, LENGTH = 0
    dram_reset      : ORIGIN = .,          LENGTH = 0
    dram_rsvd2      : ORIGIN = .,          LENGTH = 0
    dram_memory     : ORIGIN = .,          LENGTH = 32k-0x11B0 //(=0x6E50) rest is reserved as it is occupied by postloader code.
                                                          //The postloader starts always at address 0x40006E50(CAN loader) or 0x40007B00(Serial loader)
                                                          //Postloader is the same for all derivatives and it is linked to fit into the smallest micro - bolero    
}

CONSTANTS {

    stack_reserve = 8K
    //heap_reserve = 8K

}

//
// Program layout for running out of RAM.
//
SECTIONS
{    
    //Code sections
    .start                              : > dram_memory
    .text                               : > .
    .vletext                            : > .
    .syscall                            : > .
    .secinfo                            : > .
    .rodata                             : > .
    .sdata2                             : > .
    .fixaddr                            : > .
    .fixtype                            : > .

    //Data sections
    .sdabase                   ALIGN(8) : > dram_memory 
    .sdata                              : > .
    .sbss                               : > .
    .data                               : > .
    .bss                                : > .
    //.heap  ALIGN(16) PAD(heap_reserve)  : > .   
    .stack ALIGN(16) PAD(stack_reserve) : > .

    //
    // These special symbols mark the bounds of RAM and ROM memory.
    // They are used by the MULTI debugger.
    //
    __ghs_ramstart  = MEMADDR(dram_rsvd1);
    __ghs_ramend    = MEMENDADDR(dram_memory);
    __ghs_romstart  = MEMADDR(flash_rsvd1);
    __ghs_romend    = MEMENDADDR(flash_rsvd2);
    
}
