/*
    FreeRTOS V7.3.0 - Copyright (C) 2012 Real Time Engineers Ltd.

    FEATURES AND PORTS ARE ADDED TO FREERTOS ALL THE TIME.  PLEASE VISIT
    http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS tutorial books are available in pdf and paperback.        *
     *    Complete, revised, and edited pdf reference manuals are also       *
     *    available.                                                         *
     *                                                                       *
     *    Purchasing FreeRTOS documentation will not only help you, by       *
     *    ensuring you get running as quickly as possible and with an        *
     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
     *    the FreeRTOS project to continue with its mission of providing     *
     *    professional grade, cross platform, de facto standard solutions    *
     *    for microcontrollers - completely free of charge!                  *
     *                                                                       *
     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
     *                                                                       *
     *    Thank you for using FreeRTOS, and thank you for your support!      *
     *                                                                       *
    ***************************************************************************


    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    >>>NOTE<<< The modification to the GPL is included to allow you to
    distribute a combined work that includes FreeRTOS without being obliged to
    provide the source code for proprietary components outside of the FreeRTOS
    kernel.  FreeRTOS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    ***************************************************************************
     *                                                                       *
     *    Having a problem?  Start by reading the FAQ "My application does   *
     *    not run, what could be wrong?"                                     *
     *                                                                       *
     *    http://www.FreeRTOS.org/FAQHelp.html                               *
     *                                                                       *
    ***************************************************************************


    http://www.FreeRTOS.org - Documentation, training, latest versions, license
    and contact details.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool.

    Real Time Engineers ltd license FreeRTOS to High Integrity Systems, who sell
    the code with commercial support, indemnification, and middleware, under
    the OpenRTOS brand: http://www.OpenRTOS.com.  High Integrity Systems also
    provide a safety engineered and independently SIL3 certified version under
    the SafeRTOS brand: http://www.SafeRTOS.com.
*/

/* MCU specific files */
#include "common.h"
#include "mc.h"
#include "pll.h"

/* FreeRTOS includes */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/* Common Demo includes. */
#include "serial.h"
#include "BlockQ.h"
#include "blocktim.h"
#include "dynamic.h"
#include "flash.h"
#include "GenQTest.h"
#include "IntQueue.h"
#include "mevents.h"
#include "partest.h"
#include "PollQ.h"
#include "print.h"
#include "QPeek.h"
#include "semtest.h"
#include "integer.h"

/* GHS specific symbols which provides addresses of stack and data sections */
extern char __ghsbegin_sdabase[];
extern char __ghsend_stack[];
extern char __smt_s_vectStart[];
extern char __smt_h_vectStart[];

void _start( void );

//make sure the micro will execute proper code after reset 
#if (EXECUTEFROM == LOCATION_FLASH)
    
    //Set RCHW word to indicate valid boot location
    #pragma ghs section rodata=".rchw"
    const UINT32 rchwValue = 0x005A005A;    //to enable watch dog set bit 5 (WTE) to one. i.e. |0x04000000
                                            //to indicate the code is VLE code set bit 7 (VLE) to one. i.e. |0x01000000
    //pointer to the first instruction of the application code
    #pragma ghs section rodata=".jmpaddress"
    const UINT32 startAddress = (UINT32) _start;
    
    #pragma ghs section rodata=default
#endif

    //Startup code - this is the first code which is executed after reset
    //               it initalizes stack, RAM, sets MSR register and calls main()
    #pragma ghs section text=".start"
    #pragma asm
        .vle
        .section ".start", "vax"
        .globl _start
        
        _start:
            ;initialize stack
            e_lis   r1, __ghsend_stack@h
            e_or2i  r1, __ghsend_stack@l

            ;initialize RAM memory to avoid CRC exceptions 
            ;(Only used/occupied memory area is initialized)
    #ifdef INIT_RAM
            e_lis   r11, __ghsbegin_sdabase@h
            e_or2i  r11, __ghsbegin_sdabase@l
            e_lis   r12, __ghsend_stack@h
            e_or2i  r12, __ghsend_stack@l

        init_ram_loop:
            e_stmw  r0,  0(r11)
            e_addi  r11, r11, 128
            cmpw    cr0, r11, r12
            e_blt   cr0, init_ram_loop
    #endif

            ;Fill in .data and .sdata sections from their flash copy
            ;(This is valid only when locate_standalone_flash.ld and
            ; initialized variables are going to be used)
            ; It's programmer's responsibility to decide whether this is needed or not
    #ifdef INIT_RAM_VARIABLES
            ;prepare source address
            e_lis   r11, __smt_initVariablesROMCopyStart@h
            e_or2i  r11, __smt_initVariablesROMCopyStart@l

            ;prepare source end address
            e_lis   r12, __smt_initVariablesROMCopyEnd@h
            e_or2i  r12, __smt_initVariablesROMCopyEnd@l

            ;prepare destination address
            e_lis   r10, __smt_initVariablesStart@h
            e_or2i  r10, __smt_initVariablesStart@l

        ram_copy_loop:
            cmpw    cr0, r11, r12
            e_beq   cr0, ram_copy_end

            e_lbzu  r4,  0(r11)
            e_stb   r4,  0(r10)
            e_addi  r11, r11, 1
            e_addi  r10, r10, 1
            e_b     ram_copy_loop
        ram_copy_end:
    #endif
            ;jump to main function
            e_bl   main
    #pragma endasm
    #pragma ghs section text=default
/*---------------------------------------------------------------------------*/
#define IVPR                                63 // IVPR SPR number

#define mainCHECK_DELAY                     ( ( portTickType ) 5000 / portTICK_RATE_MS )
#define checkERROR_LED                      ( 3 )

/* Tests configuration */
#define testBLOCKING_QUEUE                  0
#define testBLOCK_TIME                      0
#define testDYNAMIC_PRIORITY                0
#define testGENERIC_QUEUE                   1
#define testINTEGER_MATH                    0
#define testPOLLED_QUEUE                    0
#define testQUEUE_PEEK                      0
#define testSEMAPHORE                       0
#define testLED_FLASH                       1

/* Tasks priorities */
#define mainCHECK_TASK_PRIORITY             ( tskIDLE_PRIORITY + 3 )
#define mainFLASH_TEST_TASK_PRIORITY        ( tskIDLE_PRIORITY + 2 )
#define mainBLOCK_Q_PRIORITY                ( tskIDLE_PRIORITY + 2 )
#define mainSEM_TEST_PRIORITY               ( tskIDLE_PRIORITY + 1 )
#define mainQUEUE_POLL_PRIORITY             ( tskIDLE_PRIORITY + 2 )
#define mainINTEGER_TASK_PRIORITY           ( tskIDLE_PRIORITY )
#define mainGEN_QUEUE_TASK_PRIORITY         ( tskIDLE_PRIORITY )

/*
 * The Check task periodical interrogates each of the running tests to
 * ensure that they are still executing correctly.
 * If any one of the tests fail, it is indicated with LED.
 */
void vCheckTask( void *pvParameters );

/*
 * Performs hardware modules initialization
 */
static void prvHardwareSetup( void );
/*---------------------------------------------------------------------------*/

void main( void )
{
    /* Hardware initialization code */
    prvHardwareSetup();

    /* Start the standart demo tasks */
#if testBLOCKING_QUEUE == 1
    vStartBlockingQueueTasks( mainBLOCK_Q_PRIORITY );
#endif
#if testBLOCK_TIME == 1
    vCreateBlockTimeTasks();
#endif
#if testDYNAMIC_PRIORITY == 1
    vStartDynamicPriorityTasks();
#endif
#if testGENERIC_QUEUE == 1
    vStartGenericQueueTasks( mainGEN_QUEUE_TASK_PRIORITY );
#endif
#if testINTEGER_MATH == 1
    vStartIntegerMathTasks( mainINTEGER_TASK_PRIORITY );
#endif
#if testPOLLED_QUEUE == 1
    vStartPolledQueueTasks( mainQUEUE_POLL_PRIORITY );
#endif
#if testQUEUE_PEEK == 1
    vStartQueuePeekTasks();
#endif
#if testSEMAPHORE == 1
    vStartSemaphoreTasks( mainSEM_TEST_PRIORITY );
#endif
#if testLED_FLASH == 1
    vStartLEDFlashTasks( mainFLASH_TEST_TASK_PRIORITY );
#endif

    /* Start the error checking task */
    xTaskCreate( vCheckTask, "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );

    /* Start the FreeRTOS scheduler */
    vTaskStartScheduler();

    /* Should never reach here */
    for ( ;; )
        ;
}
/*---------------------------------------------------------------------------*/

void prvHardwareSetup( void )
{
    /* Configure interrupt controller */
    INTC.MCR.R   = 0;   // set software vector mode, default table entry offset of 4 bytes
    __MTSPR( IVPR, (uint32_t)__smt_s_vectStart); // setting address of core vectors
    INTC.IACKR.R = (uint32_t)__smt_h_vectStart;  // setting address of hardware vectors
    INTC.CPR.R   = 0;   // enable priorities

    /* Disable SoftwareWatchdogTimer */
    SWT.SR.R        = 0xC520;
    SWT.SR.R        = 0xD928; // unlock peripheral
    SWT.CR.B.WEN    = 0;
    
    /* Configure RUN0 mode, with:
     a) FXOSC ON
     b) PLL ON
     c) SYSCLK=PLL
    */
    ME.RUN[0].R |= 0x74;

    /* Configure Run Peripheral Configuration register in order to active peripherals */
    ME.RUNPC[0].R   = 0x000000FE;

    /* Setup system PLL (PLL0) */
    CGM.FMPLL[0].CR.R = (FMPLL_IDF << 26 | FMPLL_ODF << 24 | FMPLL_NDIV << 16) & 0x3F7F0000;

    /* Enter RUN0 Mode */
    me_modeTransition(ME_MODE_RUN0);

    /* Initialise the LEDs */
    vParTestInitialise();
}
/*---------------------------------------------------------------------------*/

portTASK_FUNCTION( vCheckTask, pvParameters )
{
portTickType xDelay = 0;
unsigned short usErrorCode = 0;

    /* Intialise the sleeper. */
    xDelay = xTaskGetTickCount();

    for( ;; ) {
        /* Perform this check every mainCHECK_DELAY milliseconds. */
        vTaskDelayUntil( &xDelay, mainCHECK_DELAY );

        /* Check that all of the Demo tasks are still running. */
#if testBLOCKING_QUEUE == 1
        if( pdTRUE != xAreBlockingQueuesStillRunning() )
        {
            usErrorCode |= 0x1;
        }
#endif
#if testBLOCK_TIME == 1
        if( pdTRUE != xAreBlockTimeTestTasksStillRunning() )
        {
            usErrorCode |= 0x2;
        }
#endif
#if testDYNAMIC_PRIORITY == 1
        if( pdTRUE != xAreDynamicPriorityTasksStillRunning() )
        {
            usErrorCode |= 0x4;
        }
#endif
#if testGENERIC_QUEUE == 1
        if( pdTRUE != xAreGenericQueueTasksStillRunning() )
        {
            usErrorCode |= 0x8;
        }
#endif
#if testINTEGER_MATH == 1
        if( pdTRUE != xAreIntegerMathsTaskStillRunning() )
        {
            usErrorCode |= 0x10;
        }
#endif
#if testPOLLED_QUEUE == 1
        if( pdTRUE != xArePollingQueuesStillRunning() )
        {
            usErrorCode |= 0x20;
        }
#endif
#if testQUEUE_PEEK == 1
        if( pdTRUE != xAreQueuePeekTasksStillRunning() )
        {
            usErrorCode |= 0x40;
        }
#endif
#if testSEMAPHORE == 1
        if( pdTRUE != xAreSemaphoreTasksStillRunning() )
        {
            usErrorCode |= 0x80;
        }
#endif
        /* Switch LED on if errors occured */
        if( usErrorCode ) {
            vParTestSetLED( checkERROR_LED, 1 );
       }
    }
}
/*---------------------------------------------------------------------------*/
